#include "mainwindow.h"
#include "ui_mainwindow.h"

extern "C"
{
    #include "usblc3301pi.h"
}

void MainWindow::startpipethread()
{
    this->ThPipe = new PipeThread(this);
    this->ThPipe->BufSize = PIXEL_COUNT;
    connect(this->ThPipe,SIGNAL(DataChanged(quint8*,qint32,qint32,quint32)),this,SLOT(OnDataChanged(quint8*,qint32,qint32,quint32)));
    connect(this->ThPipe, SIGNAL(finished()),this->ThPipe, SLOT(deleteLater()));
    this->ThPipe->start();
}

void MainWindow::stoppipethread()
{
    if (this->ThPipe->isRunning())
    {
        this->ThPipe->stopthread();
        this->ThPipe->wait();
    }
}

void MainWindow::syncparam()
{
    quint8 u8tmp;
    qint8 i8tmp;
    quint32 u32tmp;
    quint8 uleft, ucenter, uright;

    if (ls_getstate(u8tmp,1000) == 0) {
        if (u8tmp == 0) ui->btn_start->setText("start");
        else ui->btn_start->setText("stop");
    }

    if (ls_getmode(u8tmp,1000) == 0) {
        ui->cb_mode->blockSignals(true);
        ui->cb_mode->setCurrentIndex(u8tmp);
        ui->cb_mode->blockSignals(false);
    }

    if (ls_getinttime(u32tmp,1000) == 0) {
        ui->ed_inttime->setText(QString::number(u32tmp));
    }

    if (ls_getgain(uleft,ucenter,uright,1000) == 0) {
        ui->hs_gain->blockSignals(true);
        ui->hs_gain->setValue(uleft);
        ui->hs_gain->blockSignals(false);
        ui->lb_gain->setText(QString::number(fgain[uleft],'f',2));
    }

    if (ls_getoffset(uleft,ucenter,uright,1000) == 0) {

        if (uleft >= 0x80)
        {
            i8tmp = -1 * (uleft - 0x7F);
        } else  i8tmp = uleft;

        ui->hs_offset->blockSignals(true);
        ui->hs_offset->setValue(i8tmp);
        ui->hs_offset->blockSignals(false);

        ui->lb_offset->setText(QString::number(i8tmp,'f',0));
    }

    if (ls_getcfg1(u8tmp,1000) == 0) {
        ui->sb_numframes->setValue((u8tmp & 0x0F)+1);
    }
}

void MainWindow::init_statusBar()
{
    QStatusBar * bar = ui-> statusBar;

    statusMsg = new QLabel;
    statusMsg-> setMinimumSize (350,20);
    statusMsg-> setFrameShape (QFrame :: WinPanel);
    statusMsg-> setFrameShadow (QFrame :: Sunken);

    statusFPS = new QLabel;
    statusFPS-> setMinimumSize (75,20);
    statusFPS-> setFrameShape (QFrame :: WinPanel);
    statusFPS-> setFrameShadow (QFrame :: Sunken);

    statusMEM = new QLabel;
    statusMEM-> setMinimumSize (80,20);
    statusMEM-> setFrameShape (QFrame :: WinPanel);
    statusMEM-> setFrameShadow (QFrame :: Sunken);

    bar-> addWidget (statusMsg);
    bar-> addWidget (statusFPS);
    bar-> addWidget (statusMEM);

    statusMsg->setText ("TinyUSB Line Camera - TSL3301");
    statusFPS->setText ("FPS: 0");
    statusMEM->setText ("MEM: 0");
}

void MainWindow::create_chart()
{
    plot = new QwtPlot(ui->plotwidget);
    plot->setTitle( "TinyUSB Line Camera - TSL3301" );
    plot->setCanvasBackground( Qt::white );
    plot->setAxisScale( QwtPlot::yLeft, 0, 255);
    plot->setAxisScale( QwtPlot::xBottom, 0, PIXEL_COUNT-1);

    grid = new QwtPlotGrid();
    grid->attach(plot);

    curve = new QwtPlotCurve("Curve 1");
    curve->setPen( Qt::blue, 0 ),
    curve->setRenderHint( QwtPlotItem::RenderAntialiased, true );

    seriesdata = new QwtPointSeriesData;
    points = new QVector<QPointF>;

    for(int i=0;i<PIXEL_COUNT;i++){
        points->push_back(QPointF(i,0));
    }
    seriesdata->setSamples(*points);
    curve->setData(seriesdata);
    curve->attach(plot);

    plot->resize( 500, 320 );
    plot->replot();
}

void MainWindow::OnDataChanged(quint8* buf, qint32 BytesRead, qint32 BytesAvailable, quint32 fps)
{
    statusMEM->setText("MEM: "+QString::number(BytesAvailable));
    statusFPS->setText("FPS: "+QString::number(fps));

    points->clear();
    for(int i=0; i<BytesRead;i++){
        points->push_back(QPointF(i,buf[i]));
    }
    seriesdata->setSamples(*points);

    //replot() is too slow..
    //10 ms refresh / replot rate
    if (PlotRefreshTimer.elapsed() > 10) {
        plot->replot();
        PlotRefreshTimer.restart();
    }
}

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    init_statusBar();

    this->packetlength = PIXEL_COUNT * PACKETLENGTH_MULTIPLIER;
    ls_initialize(4*1024*1024,this->packetlength); //4MB PipeSize
    create_chart();

    PlotRefreshTimer.start();
}

MainWindow::~MainWindow()
{
    close_device();
    delete ui;
}

void MainWindow::close_device()
{
    if (ls_currentdeviceindex()>= 0){
        this->stoppipethread();
        ls_closedevice();
        ui->btn_open->setText("open");
    }
}

void MainWindow::open_device(qint32 index)
{
    qint32 tmp_packetlength;
    qint32 ierr;

    if (ls_currentdeviceindex()>= 0){
        close_device();
    } else {
        if (ls_devicecount() > 0){
            ierr = ls_opendevicebyindex(index);
            if (ierr == 0){
                ierr = ls_getpacketlength(tmp_packetlength,1000);
                if (ierr == 0) {
                    if (tmp_packetlength != (this->packetlength / PACKETLENGTH_MULTIPLIER)) {
                        this->packetlength = (tmp_packetlength * PACKETLENGTH_MULTIPLIER);
                        ls_closedevice();
                        ls_setpacketlength(this->packetlength);
                        ierr = ls_opendevicebyindex(index);
                        if (ierr != 0) {
                            statusMsg->setText("OPEN DEVICE: "+QString(ls_geterrorstring(ierr)));
                            return;
                        }
                    }
                } else statusMsg->setText("GET PACKETLENGTH: "+QString(ls_geterrorstring(ierr)));


                this->syncparam();
                this->startpipethread();
                ui->btn_open->setText("close");
            } else statusMsg->setText("OPEN DEVICE: "+QString(ls_geterrorstring(ierr)));
        }
    }
}

void MainWindow::on_btn_enum_clicked()
{
    quint16 cnt;

    this->close_device();
    ui->cb_devices->clear();
    cnt = ls_enumdevices();
    if (cnt > 0) {
        statusMsg->setText("ENUM DEVICES: "+QString::number(cnt)+" DEVICE(S) FOUND");
        for(int i=0;i<cnt;i++){
            ui->cb_devices->addItem(QString(ls_getproductname(i))+" "+QString(ls_getserialnumber(i)));
        }
        ui->cb_devices->setCurrentIndex(0);
        this->open_device(ui->cb_devices->currentIndex());
    } else statusMsg->setText("ENUM DEVICES: NO DEVICES FOUND");
}

void MainWindow::on_btn_open_clicked()
{
    this->open_device(ui->cb_devices->currentIndex());
}

void MainWindow::on_btn_start_clicked()
{
    quint8 state;
    quint8 mode;
    qint32 ierr;

    ierr = ls_getstate(state,1000);
    if (ierr == 0){
        state = (state & 0x01) ^ 0x01;
        ierr = ls_setstate(state,1000);
        if (ierr == 0){
            ls_getmode(mode,1000);
            if ((state == 0) || (mode == 0)) ui->btn_start->setText("start");
            else ui->btn_start->setText("stop");
        }
    }
    statusMsg->setText("SET STATE: "+QString(ls_geterrorstring(ierr)));
}

void MainWindow::on_btn_inttime_clicked()
{
    qint32 ierr;
    quint32 u32tmp;

    ierr = ls_setinttime(ui->ed_inttime->text().toInt(),1000);
    if (ierr == 0) {
        if (ls_getinttime(u32tmp,1000) == 0) {
            ui->ed_inttime->setText(QString::number(u32tmp));
        }
    }
    statusMsg->setText("SET INT.TIME: "+QString(ls_geterrorstring(ierr)));
}

void MainWindow::on_hs_gain_valueChanged(int value)
{
    qint32 ierr;
    quint8 ucgain;

    ucgain = value;
    ierr = ls_setgain(ucgain,ucgain,ucgain,1000);
    statusMsg->setText("SET GAIN: "+QString(ls_geterrorstring(ierr)));

    ui->lb_gain->setText(QString::number(fgain[ucgain],'f',2));
}

void MainWindow::on_hs_offset_valueChanged(int value)
{
    qint32 ierr;
    quint8 ucoffset;

    if (value < 0)
    {
      ucoffset = abs(value);
      ucoffset = ucoffset + 0x7F;
    } else ucoffset = value;

    ierr = ls_setoffset(ucoffset,ucoffset,ucoffset,1000);
    statusMsg->setText("SET OFFSET: "+QString(ls_geterrorstring(ierr)));

    ui->lb_offset->setText(QString::number(value,'f',0));
}

void MainWindow::on_cb_mode_currentIndexChanged(int index)
{
    qint32 ierr;

    ierr = ls_setmode(index,1000);
    if ((ierr == 0) && (index == 0))
        ui->btn_start->setText("start");

    statusMsg->setText("SET MODE: "+QString(ls_geterrorstring(ierr)));
}

void MainWindow::on_btn_savesettings_clicked()
{
    qint32 ierr;

    ierr = ls_savesettings(1000);
    statusMsg->setText("SAVE SETTINGS: "+QString(ls_geterrorstring(ierr)));
}

void MainWindow::on_btn_getinfo_clicked()
{
    qint32 ierr;
    quint16 u16tmp;
    qint32 i32tmp;
    quint32 u32tmp;
    int idx;

    idx = ui->cb_devices->currentIndex();

    ui->lb_vendor->setText(ls_getvendorname(idx));
    ui->lb_product->setText(ls_getproductname(idx));
    ui->lb_serial->setText(ls_getserialnumber(idx));

    u16tmp = ls_getfwversion(idx);
    ui->lb_mcu1version->setText(QString::number((u16tmp >> 8) & 0xFF)+"."+QString::number(u16tmp & 0xFF));

    ui->lb_sensor->setText(ls_getsensorname());
    ui->lb_pixel->setText(QString::number(ls_getpixelcount()));

    ierr = ls_getpacketlength(i32tmp,1000);
    if (ierr == 0) ui->lb_mcu2packet->setText(QString::number(i32tmp)+" Bytes");
    else ui->lb_mcu2packet->setText("?0");

    ierr = ls_customfirmware(u32tmp,1000);
    if (ierr == 0) ui->lb_custom->setText(QString::number(u32tmp));
    else ui->lb_custom->setText("?0");

    u16tmp = ls_libversion();
    ui->lb_libversion->setText(QString::number((u16tmp >> 8) & 0xFF)+"."+QString::number(u16tmp & 0xFF));  //+QString::number(u16tmp & 0xFF,16));
}

void MainWindow::on_pushButton_clicked()
{
    qint32 ierr;
    quint8 uccfg1;

    uccfg1 = (ui->sb_numframes->value() & 0x0F)-1;
    ierr = ls_setcfg1(uccfg1,1000);
    if (ierr == 0)
    {
      //Changes takes effect after reboot - Power OFF/ON
    }
    statusMsg->setText("SET CFG1: "+QString(ls_geterrorstring(ierr)));
}
