#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QLabel>
#include <qwt.h>
#include <qwt_plot_curve.h>
#include <qwt_plot.h>
#include <qwt_plot_grid.h>
#include <qwt_symbol.h>
#include <qwt_legend.h>
#include "pipethread.h"

#define PACKETLENGTH_MULTIPLIER 1
#define PIXEL_COUNT             102

const float fgain[32] = {1.00, 1.02, 1.05, 1.07, 1.09, 1.12, 1.15, 1.18, 1.21, 1.24, 1.27, 1.31,
                         1.34, 1.38, 1.43, 1.47, 1.52, 1.57, 1.62, 1.68, 1.74, 1.81, 1.88, 1.96,
                         2.05, 2.14, 2.24, 2.35, 2.48, 2.61, 2.77, 2.94};


namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();
    PipeThread *ThPipe;

private slots:
    void on_btn_enum_clicked();

    void on_btn_open_clicked();

    void on_btn_start_clicked();

    void on_btn_inttime_clicked();

    void on_hs_gain_valueChanged(int value);

    void on_hs_offset_valueChanged(int value);

    void on_cb_mode_currentIndexChanged(int index);

    void on_btn_savesettings_clicked();

    void on_btn_getinfo_clicked();

    void on_pushButton_clicked();

private:
    Ui::MainWindow *ui;
    QwtPlot *plot;
    QwtPlotGrid *grid;
    QwtPlotCurve *curve;
    QwtPointSeriesData* seriesdata;
    QVector<QPointF>* points;

    qint32 packetlength;

    QLabel * statusMsg;
    QLabel * statusFPS;
    QLabel * statusMEM;

    QTime PlotRefreshTimer;

    void init_statusBar ();
    void create_chart();

    void close_device();
    void open_device(qint32);
    void startpipethread();
    void stoppipethread();
    void syncparam();

public slots:
    void OnDataChanged(quint8*,qint32,qint32,quint32);
};

#endif // MAINWINDOW_H
